import { Column, Entity, ManyToMany, Relation } from 'typeorm';

import { CommonEntity } from '~/common/entity/common.entity';

import { RoleEntity } from '../role/role.entity';

@Entity({ name: 'sys_menu' })
export class MenuEntity extends CommonEntity {
  @Column({ name: 'parent_id', nullable: true, comment: '父级ID' })
  parentId: number;

  @Column()
  name: string;

  @Column({ nullable: true, comment: '前端路径' })
  path: string;

  @Column({ nullable: true, comment: '权限' })
  permission: string;

  @Column({ type: 'tinyint', default: 0, comment: '类型：0-目录 1-菜单 2-权限' })
  type: number;

  @Column({ nullable: true, default: '', comment: '图标' })
  icon: string;

  @Column({ name: 'order_no', type: 'int', nullable: true, default: 0 })
  orderNo: number;

  @Column({ name: 'component', nullable: true, comment: '前端组件文件地址' })
  component: string;

  @Column({ name: 'is_ext', type: 'boolean', default: false })
  isExt: boolean;

  @Column({ name: 'ext_open_mode', type: 'tinyint', default: 1 })
  extOpenMode: number;

  @Column({ name: 'keep_alive', type: 'tinyint', default: 1 })
  keepAlive: number;

  @Column({ type: 'tinyint', default: 1 })
  show: number;

  @Column({ name: 'active_menu', nullable: true })
  activeMenu: string;

  @Column({ type: 'tinyint', default: 1 })
  status: number;

  @Column({ type: 'tinyint', default: 1,comment: '用户端类型：0-APP 1-PC' })
  device: number;

  @ManyToMany(() => RoleEntity, role => role.menus, {
    onDelete: 'CASCADE'
  })
  roles: Relation<RoleEntity[]>;
}
